@@ -11,6 +11,7 @@
 #include <QFile>
 #include <QJsonArray>
 #include <QJsonDocument>
+#include <QJsonObject>
 #include <QStandardPaths>
 #include <QUrl>
 
@@ -28,6 +29,11 @@ TabsModel::TabsModel(QObject *parent)
     m_tabsReadOnly = true;
     // Make sure model always contains at least one tab
     createEmptyTab();
+
+    // Only load tabs after private mode is known
+    connect(this, &TabsModel::privateModeChanged, [this] {
+        loadInitialTabs();
+    });
 }
 
 QHash<int, QByteArray> TabsModel::roleNames() const
@@ -35,6 +41,7 @@ QHash<int, QByteArray> TabsModel::roleNames() const
     return {
         {RoleNames::UrlRole, QByteArrayLiteral("pageurl")},
         {RoleNames::IsMobileRole, QByteArrayLiteral("isMobile")},
+        {RoleNames::IsDeveloperToolsOpen, QByteArrayLiteral("isDeveloperToolsOpen")},
     };
 }
 
@@ -49,6 +56,8 @@ QVariant TabsModel::data(const QModelIndex &index, int role) const
         return m_tabs.at(index.row()).url();
     case RoleNames::IsMobileRole:
         return m_tabs.at(index.row()).isMobile();
+    case RoleNames::IsDeveloperToolsOpen:
+        return m_tabs.at(index.row()).isDeveloperToolsOpen();
     }
 
     return {};
@@ -81,6 +90,10 @@ TabState TabsModel::tab(int index)
  */
 void TabsModel::loadInitialTabs()
 {
+    if (m_initialTabsLoaded) {
+        return;
+    }
+
     if (!m_privateMode) {
         loadTabs();
     }
@@ -98,6 +111,8 @@ void TabsModel::loadInitialTabs()
                 newTab(BrowserManager::instance()->initialUrl());
         }
     }
+
+    m_initialTabsLoaded = true;
 }
 
 /**
@@ -321,6 +336,27 @@ void TabsModel::setIsMobile(int index, bool isMobile)
     saveTabs();
 }
 
+void TabsModel::setIsDeveloperToolsOpen(int index, bool isDeveloperToolsOpen)
+{
+    qDebug() << "Setting isDeveloperToolsOpen:" << index << isDeveloperToolsOpen << "tabs open" << m_tabs.count();
+    if (index < 0 && index >= m_tabs.count())
+        return; // index out of bounds
+
+    m_tabs[index].setIsDeveloperToolsOpen(isDeveloperToolsOpen);
+
+    const QModelIndex mindex = createIndex(index, index);
+    emit dataChanged(mindex, mindex, {RoleNames::IsDeveloperToolsOpen});
+    saveTabs();
+}
+
+bool TabsModel::isDeveloperToolsOpen(int index)
+{
+    if (index < 0 && index >= m_tabs.count())
+        return false;
+
+    return m_tabs.at(index).isDeveloperToolsOpen();
+}
+
 void TabsModel::setUrl(int index, const QString &url)
 {
     qDebug() << "Setting URL:" << index << url << "tabs open" << m_tabs.count();
@@ -354,11 +390,22 @@ void TabState::setIsMobile(bool isMobile)
     m_isMobile = isMobile;
 }
 
+bool TabState::isDeveloperToolsOpen() const
+{
+    return m_isDeveloperToolsOpen;
+}
+
+void TabState::setIsDeveloperToolsOpen(bool isDeveloperToolsOpen)
+{
+    m_isDeveloperToolsOpen = isDeveloperToolsOpen;
+}
+
 TabState TabState::fromJson(const QJsonObject &obj)
 {
     TabState tab;
     tab.setUrl(obj.value(QStringLiteral("url")).toString());
     tab.setIsMobile(obj.value(QStringLiteral("isMobile")).toBool());
+    tab.setIsDeveloperToolsOpen(obj.value(QStringLiteral("isDeveloperToolsOpen")).toBool());
     return tab;
 }
 
@@ -370,7 +417,11 @@ TabState::TabState(const QString &url, const bool isMobile)
 
 bool TabState::operator==(const TabState &other) const
 {
-    return (m_url == other.url() && m_isMobile == other.isMobile());
+    return (
+        m_url == other.url() &&
+        m_isMobile == other.isMobile() &&
+        m_isDeveloperToolsOpen == other.isDeveloperToolsOpen()
+    );
 }
 
 QJsonObject TabState::toJson() const
@@ -378,5 +429,6 @@ QJsonObject TabState::toJson() const
     return {
         {QStringLiteral("url"), m_url},
         {QStringLiteral("isMobile"), m_isMobile},
+        {QStringLiteral("isDeveloperToolsOpen"), m_isDeveloperToolsOpen},
     };
 }
